/*******************************************************************************
 * Copyright (c) 2011 INRIA.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Contributors:
 * Cau Clasen - initial API and implementation
 *******************************************************************************/

package fr.inria.virtualemf.core;

import java.util.ArrayList;
import java.util.List;

import org.eclipse.emf.ecore.EObject;
import org.eclipse.emf.ecore.EStructuralFeature;
import org.eclipse.emf.ecore.resource.Resource;

import fr.inria.virtualemf.elements.FilterElement;
import fr.inria.virtualemf.elements.MergeElementImpl;
import fr.inria.virtualemf.elements.ReproduceElementImpl;
import fr.inria.virtualemf.elements.VirtualElement;
import fr.inria.virtualemf.rules.TranslationRule;

public class CorrespondenceModelAMWProjector {

	private VirtualModel virtualModel;
	private Resource leftModel;
	private Resource rightModel;

	public CorrespondenceModelAMWProjector(VirtualModel vModel){

		this.virtualModel= vModel;
		leftModel = vModel.getContributingModels().get(0);
		rightModel = vModel.getContributingModels().get(1);

	}

	@SuppressWarnings("unchecked")
	public void load(Resource correspondenceModel){

		List<EObject> associations = new ArrayList<EObject>();
		List<EObject> merges = new ArrayList<EObject>();
		List<EObject> filters = new ArrayList<EObject>();
		EObject virtualComposition = correspondenceModel.getContents().get(0);
		for (EObject eo : (List<EObject>) virtualComposition.eGet(VirtualModelUtil.getFeatureFromName(virtualComposition, "ownedElement"))){
			if(eo.eClass().getName().equals("Association")){
				associations.add(eo);
			} else if(eo.eClass().getName().equals("Merge")){
				merges.add(eo);
			} else if (eo.eClass().getName().equals("Filter")){
				filters.add(eo);
			}
		}
		loadAssociations(associations);
		loadMerges(merges);
		loadFilters(filters);
	}

	private void loadFilters(List<EObject> filters) {
		for (EObject link : filters) {
			EObject end = (EObject) link.eGet(VirtualModelUtil.getFeatureFromName(link, "filteredElement"));
			EObject filteredElement = getReferencedObject(end);
			FilterElement filter = new FilterElement(filteredElement);
			virtualModel.getVirtualLinkManager().setVirtualLink(filteredElement, filter);
		}
	}

	@SuppressWarnings("unchecked")
	private void loadAssociations(List<EObject> associations) {
		for(EObject vLink : associations){

			EObject sourceEnd = (EObject)vLink.eGet(VirtualModelUtil.getFeatureFromName(vLink, "sourceElement"));
			EObject sourceElement = getReferencedObject(sourceEnd);

			List<EObject> targetElements = new ArrayList<EObject>();
			for (EObject targetEnd : (List<EObject>)vLink.eGet(VirtualModelUtil.getFeatureFromName(vLink, "targetElement"))) {
				targetElements.add(getReferencedObject(targetEnd));
			}

			ReproduceElementImpl vElement = (ReproduceElementImpl) virtualModel.getVirtualLinkManager().getVirtualElement(sourceElement);

			String virtualFeatureName = (String) vLink.eGet(VirtualModelUtil.getFeatureFromName(vLink, "associationTypeName"));
			EStructuralFeature virtualFeature = virtualModel.getMetamodelManager().getVirtualAssociation(vElement, virtualFeatureName);

			// only reproduced elements can have virtual associations
			vElement.setVirtualAssociation(virtualFeature, TranslationRule.NO_INDEX, targetElements);
			
		}
	}
	
	@SuppressWarnings("unchecked")
	private void loadMerges(List<EObject> merges) {
		for (EObject vLink : merges) {

			List<EObject> mergingElements = new ArrayList<EObject>();
			List<EObject> mergedElementsEnd = (List<EObject>) vLink.eGet(VirtualModelUtil.getFeatureFromName(vLink, "mergedElements"));
			for (EObject mergeEnd : mergedElementsEnd){
				mergingElements.add(getReferencedObject(mergeEnd));
			}
			
			EObject prefEnd = (EObject) vLink.eGet(VirtualModelUtil.getFeatureFromName(vLink, "preferableElement"));
			String mergeClassName = (String) vLink.eGet(VirtualModelUtil.getFeatureFromName(vLink, "mergedClassName"));
			
			EObject preferableElement;
			if(prefEnd == null){
				preferableElement = mergingElements.get(0);
			} else {
				preferableElement = getReferencedObject(prefEnd);
			}
			
			VirtualElement mergedElement = new MergeElementImpl(virtualModel, mergingElements, preferableElement, mergeClassName);
			for (EObject eo : mergingElements) {
				virtualModel.getVirtualLinkManager().setVirtualLink(eo, mergedElement);
			}
		}
	}

	private EObject getReferencedObject(EObject end){
		EObject ref = (EObject) end.eGet(VirtualModelUtil.getFeatureFromName(end, "element"));
		String id = (String) ref.eGet(VirtualModelUtil.getFeatureFromName(ref, "ref"));
		if(ref.eContainer().eGet(VirtualModelUtil.getFeatureFromName(ref, "name")).equals("leftModel")){
			return leftModel.getEObject(id);
		} else {
//			for (Iterator iterator = rightModel.getAllContents(); iterator.hasNext();) {
//				EObject n = (EObject) iterator.next();
//				if(n.eClass().getName().equals("Class")){
//					Resource res = n.eResource();
//					String idd = res.getURIFragment(n);
//					System.out.println(idd);
//				}
//				
//			}
			return rightModel.getEObject(id);
		}
	}
	
	public void save(Resource correspondenceModel){
		
	}

}
