/*******************************************************************************
 * Copyright (c) 2011 INRIA.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Contributors:
 * Cau Clasen - initial API and implementation
 *******************************************************************************/

package fr.inria.virtualemf.core;

import java.util.HashMap;
import java.util.Map;

import org.eclipse.emf.ecore.EObject;
import org.eclipse.emf.ecore.resource.Resource;

import fr.inria.virtualemf.elements.ReproduceElementImpl;
import fr.inria.virtualemf.elements.VirtualElement;
import fr.inria.virtualemf.vlink_mm.VirtualLinksMetamodel;

public class VirtualLinkManager {

	private VirtualModel virtualModel;
	//	private ResourceSet resourceSet;
	//	private Resource leftModel;
	//	private Resource rightModel;
	private Resource correspondenceModel;
	//	private Resource correspondenceMetamodel;
	private Map<EObject, VirtualElement> virtualLinks = new HashMap<EObject, VirtualElement>();

	public VirtualLinkManager(String correspondenceModelPath, VirtualModel vModel) {
		this.virtualModel = vModel;
		Resource correspondenceMetamodel = VirtualLinksMetamodel.getVirtualLinksMetamodel(vModel.getVirtualResourceSet());
		VirtualModelUtil.registerMetamodel(correspondenceMetamodel, vModel.getVirtualResourceSet());
		Resource.Factory.Registry.INSTANCE.getExtensionToFactoryMap().put("amw", Resource.Factory.Registry.INSTANCE.getExtensionToFactoryMap().get("xmi"));
		correspondenceModel = VirtualModelUtil.loadResource(vModel, correspondenceModelPath);
	}

	public void initialize(){
		CorrespondenceModelAMWProjector projector = new CorrespondenceModelAMWProjector(virtualModel);
		projector.load(correspondenceModel);
	}

	public void setVirtualLink(EObject concreteElement, VirtualElement virtualElement){
		virtualLinks.put(concreteElement, virtualElement);
	}
	
	public Map<EObject, VirtualElement> getVirtualLinks(){
		return virtualLinks;
	}

	public boolean isVirtuallyLinked(EObject concreteElement){
		return virtualLinks.containsKey(concreteElement);
	}

	public EObject getVirtualElement(EObject e) {
		VirtualElement vElem = virtualLinks.get(e);
		if(vElem == null){
			vElem = new ReproduceElementImpl(virtualModel, e);
			virtualLinks.put(e, vElem);
		}
		return vElem;
	}

	public Resource getCorrespondenceModel() {
		return correspondenceModel;
	}

	public void setCorrespondenceModel(Resource correspondenceModel) {
		this.correspondenceModel = correspondenceModel;
	}

	public void save() {
		CorrespondenceModelAMWProjector projector = new CorrespondenceModelAMWProjector(virtualModel);
		projector.save(correspondenceModel);
	}

}
