/*******************************************************************************
 * Copyright (c) 2011 INRIA.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Contributors:
 * Cau Clasen - initial API and implementation
 *******************************************************************************/

package fr.inria.virtualemf.core;

import java.util.Iterator;

import org.eclipse.emf.common.util.URI;
import org.eclipse.emf.ecore.EObject;
import org.eclipse.emf.ecore.EPackage;
import org.eclipse.emf.ecore.EStructuralFeature;
import org.eclipse.emf.ecore.resource.Resource;
import org.eclipse.emf.ecore.resource.ResourceSet;

public class VirtualModelUtil {

	public static URI toURI(Resource resource, String path) {
		return URI.createURI(resource.getURI().toString().replaceAll("/[^/]*$", "/" + path));
	}

	public static EStructuralFeature getFeatureFromName(EObject object, String featureName){
		return object.eClass().getEStructuralFeature(featureName);
	}

	// performs a structural comparison of features. Check if they have the same name and type name
	public static boolean compareFeatures(EStructuralFeature left, EStructuralFeature right){
		return left.getName().equals(right.getName()) && left.getEType().getName().equals(right.getEType().getName());
	}

	public static void registerMetamodel(Resource metamodel, ResourceSet resourceSet){
		for(Iterator<EObject> i = metamodel.getAllContents(); i.hasNext();) {
			EObject eo = i.next();
			if(eo instanceof EPackage) {
				resourceSet.getPackageRegistry().put(((EPackage)eo).getNsURI(), eo);
			}
		}
	}

	// searches the resource from its filename in the same folder of the virtual model
	public static Resource loadResource(VirtualModel vModel, String pathName){
		return vModel.getVirtualResourceSet().getResource(URI.createPlatformResourceURI(pathName, true), true);
//		return vModel.getVirtualResourceSet().getResource(toURI(vModel, fileName), true);
	}

}
