/*******************************************************************************
 * Copyright (c) 2011 INRIA.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Contributors:
 * Cau Clasen - initial API and implementation
 *******************************************************************************/

package fr.inria.virtualemf.elements;

import java.util.ArrayList;
import java.util.List;

import org.eclipse.emf.common.util.EList;
import org.eclipse.emf.ecore.EClass;
import org.eclipse.emf.ecore.EObject;
import org.eclipse.emf.ecore.EStructuralFeature;

import fr.inria.virtualemf.core.VirtualModel;
import fr.inria.virtualemf.core.VirtualModelOperationException;
import fr.inria.virtualemf.rules.MergeRule;
import fr.inria.virtualemf.rules.VirtualModelList;

public class MergeElementImpl extends VirtualElement {

	// Element used to decide in conflicting cases (e.g. properties with same name, container...)
	private EObject preferableElement;
	private List<EObject> concreteElements = new ArrayList<EObject>();
	private EClass mergeClass = null;

	public MergeElementImpl(VirtualModel vModel, List<EObject> concreteElements, EObject preferableElement, String mergeClassName){
		super();
		mergeClass = vModel.getMetamodelManager().getMergeClass(mergeClassName);
		init(vModel, concreteElements, preferableElement, mergeClass);
	}

	public MergeElementImpl(VirtualModel vModel, List<EObject> concreteElements, EClass eClass, EObject preferableElement){
		super();
		init(vModel, concreteElements, preferableElement, eClass);
	}
	
	private void init(VirtualModel vModel, List<EObject> concreteElements, EObject preferableElement, EClass eClass) {
		if (concreteElements.size() != 2)
			System.out.println();
		this.eProperties().setEResource(vModel);
		this.concreteElements = concreteElements;
		setTranslationRule(MergeRule.INSTANCE);
		this.eSetClass(eClass);
		eSetStore(this.getTranslationRule());
		setPreferableElement(preferableElement);
		List<EClass> ecs = new ArrayList<EClass>();
		for (EObject eo : concreteElements) {
			try {
				ecs.add(eo.eClass());
			} catch (Exception e) {
				// TODO Auto-generated catch block
				e.printStackTrace();
			}
		}
	}

	public EObject getPreferableElement() {
		return preferableElement;
	}

	public void setPreferableElement(EObject object) {
		this.preferableElement = object;
	}

	// this should only be called for single valued features
	public EObject getContainmentElement(EStructuralFeature concreteFeature){
		for (EObject obj : this.concreteElements) {
			for (EStructuralFeature sf : obj.eClass().getEAllStructuralFeatures()) {
				if (sf == concreteFeature){
					return obj;
				}
			}
		}
		throw new VirtualModelOperationException("concrete element could not found");
	}

	@SuppressWarnings("unchecked")
	public List<EObject> getMergedList(EStructuralFeature feature){
		List<EStructuralFeature> concFeatures = ((VirtualModel)eResource()).getMetamodelManager().translateFromMergedFeature(this, feature);
		EList<EObject>[] subLists = new EList[concreteElements.size()];
		for (int i = 0; i < subLists.length; i++) {
			EStructuralFeature csf = concFeatures.get(i);
			EObject ceo = getContainmentElement(csf);
			subLists[i] = (EList<EObject>) ceo.eGet(csf);
		}
		return new VirtualModelList<EObject>(this, feature, subLists);
	}
	
	public List<EObject> getConcreteElements(){
		return concreteElements;
	}

}
