/*******************************************************************************
 * Copyright (c) 2011 INRIA.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Contributors:
 * Cau Clasen - initial API and implementation
 *******************************************************************************/

package fr.inria.virtualemf.rules;

import java.util.ArrayList;
import java.util.List;

import org.eclipse.emf.common.util.EList;
import org.eclipse.emf.ecore.EClass;
import org.eclipse.emf.ecore.EObject;
import org.eclipse.emf.ecore.EReference;
import org.eclipse.emf.ecore.EStructuralFeature;
import org.eclipse.emf.ecore.InternalEObject;

import fr.inria.virtualemf.core.VirtualModel;
import fr.inria.virtualemf.core.VirtualModelOperationException;
import fr.inria.virtualemf.elements.MergeElementImpl;
import fr.inria.virtualemf.elements.ReproduceElementImpl;

public class MergeRule extends TranslationRule {

	public static TranslationRule INSTANCE = new MergeRule();

	@SuppressWarnings("unchecked")
	public Object get(InternalEObject object, EStructuralFeature feature, int index) {
		MergeElementImpl mergeElement = (MergeElementImpl)object;
		VirtualModel vModel = (VirtualModel) object.eResource();
		List<EStructuralFeature> cFeatures = vModel.getMetamodelManager().translateFromMergedFeature(mergeElement, feature);
		Object value = null;
		if(feature instanceof EReference){
			if(feature.isMany()) {
				if(cFeatures.size() > 1){
					value = ((MergeElementImpl)object).getMergedList(feature); 
				} else {
					value = mergeElement.getContainmentElement(cFeatures.get(0)).eGet(cFeatures.get(0));
					value = new VirtualModelList<EObject>(object, feature, (EList<EObject>)value);
				}
				if (index  != NO_INDEX) {
					value = ((VirtualModelList<EObject>)value).get(index);
				}
			} else {
				if(cFeatures.size() > 1){
					EObject preferable = mergeElement.getPreferableElement();
					for (EStructuralFeature concFeat : cFeatures) {
						if(mergeElement.getContainmentElement(concFeat) == preferable){
							value = vModel.translateToVirtualElement((EObject) preferable.eGet(concFeat));
						}
					}
				} else {
					value = mergeElement.getContainmentElement(cFeatures.get(0)).eGet(cFeatures.get(0));
					value = vModel.translateToVirtualElement((EObject) value);
				}
			}
		} else {
			if(feature.isMany()){
				if(cFeatures.size() > 1){
					value = new ArrayList<Object>();
					for (EStructuralFeature concFeat : cFeatures) {
						EObject concObj = mergeElement.getContainmentElement(concFeat);
						List<Object> list = (List<Object>) concObj.eGet(concFeat);
						((List<Object>)value).addAll(list);
					}
				} else {
					value = mergeElement.getContainmentElement(cFeatures.get(0)).eGet(cFeatures.get(0));
				}
				if(index != NO_INDEX){
					value = ((List<Object>)value).get(index);
				}
			} else {
				if(cFeatures.size() > 1){
					EObject preferable = mergeElement.getPreferableElement();
					for (EStructuralFeature concFeat : cFeatures) {
						if(mergeElement.getContainmentElement(concFeat) == preferable){
							value = preferable.eGet(concFeat);
						}
					}
				} else {
					value = mergeElement.getContainmentElement(cFeatures.get(0)).eGet(cFeatures.get(0));
				}
			}
		}
		return value;
	}

	@SuppressWarnings("unchecked")
	@Override
	public Object set(InternalEObject object, EStructuralFeature feature, int index, Object value) {
		MergeElementImpl mergeElement = (MergeElementImpl)object;
		VirtualModel vModel = (VirtualModel) object.eResource();
		List<EStructuralFeature> cFeatures = vModel.getMetamodelManager().translateFromMergedFeature(mergeElement, feature);
		List<EObject> cElements = ((MergeElementImpl)object).getConcreteElements();
		if(feature instanceof EReference){
			if(feature.isMany()) {
				if (index  != NO_INDEX) {
					if(value instanceof MergeElementImpl){ // this will probably never happen
						value = ((MergeElementImpl) value).getPreferableElement();
					} else if(value instanceof ReproduceElementImpl){
						value = ((ReproduceElementImpl)value).getConcreteElement();
					}
					if(cFeatures.size() > 1) {
						for (int i = 0; i < cFeatures.size(); i++) {
							((MergeElementImpl)object).getConcreteElements().get(i).eSet(cFeatures.get(i), value);
						}
					} else { // == 1
						((MergeElementImpl)object).getContainmentElement(cFeatures.get(0)).eSet(cFeatures.get(0), value);
					}
				} else {
					if(value instanceof MergeElementImpl){ // this will probably never happen
						value = ((MergeElementImpl) value).getPreferableElement();
					} else if(value instanceof ReproduceElementImpl){
						value = ((ReproduceElementImpl)value).getConcreteElement();
					}
					if(cFeatures.size() > 1) {
						for (int i = 0; i < cFeatures.size(); i++) {
							((EList<EObject>)((MergeElementImpl)object).getConcreteElements().get(i).eGet(cFeatures.get(i))).set(index, (EObject) value);
						}
					} else { // == 1
						((EList<EObject>)((MergeElementImpl)object).getContainmentElement(cFeatures.get(0)).eGet(cFeatures.get(0))).set(index, (EObject) value);
					}
				}
			} else {
				if(value instanceof MergeElementImpl){
					value = ((MergeElementImpl) value).getPreferableElement();
				} else if(value instanceof ReproduceElementImpl) {
					value = ((ReproduceElementImpl)value).getConcreteElement();
				}
				if(cFeatures.size() > 1) {
					for (int i = 0; i < cFeatures.size(); i++) {
						((MergeElementImpl)object).getConcreteElements().get(i).eSet(cFeatures.get(i), value);
					}
				} else {
					((MergeElementImpl)object).getConcreteElements().get(0).eSet(cFeatures.get(0), value);
				}
			}
		} else {
			if(feature.isMany()){
				if(index == NO_INDEX){
					if(cFeatures.size() > 1) {
						for (int i = 0; i < cFeatures.size(); i++) {
							((MergeElementImpl)object).getConcreteElements().get(i).eSet(cFeatures.get(i), value);
						} 
					} else {
						((MergeElementImpl)object).getConcreteElements().get(0).eSet(cFeatures.get(0), value);
					}
				} else {				
					if(cFeatures.size() > 1) {
						for (int i = 0; i < cFeatures.size(); i++) {
							((EList<Object>)((MergeElementImpl)object).getConcreteElements().get(i).eGet(cFeatures.get(i))).set(index, value);
						}
					} else { // don't need this else
						((EList<Object>)((MergeElementImpl)object).getConcreteElements().get(0).eGet(cFeatures.get(0))).set(index, value);
					}
				}
			} else {
				for (int i = 0; i < cFeatures.size(); i++) {
					((MergeElementImpl)object).getConcreteElements().get(i).eSet(cFeatures.get(i), value);
				}
			}
		}
		return null; //should return old value
	}

	@Override
	public boolean isSet(InternalEObject object, EStructuralFeature feature) {
		try {
			MergeElementImpl mergeElement = (MergeElementImpl)object;
			VirtualModel vModel = (VirtualModel) mergeElement.eResource();
			List<EStructuralFeature> concFeatures = vModel.getMetamodelManager().translateFromMergedFeature(mergeElement, feature);
			boolean ret = false;
			for (EStructuralFeature concFeature : concFeatures) {
				EObject obj = mergeElement.getContainmentElement(concFeature);
				ret = obj.eIsSet(concFeature);
				if(ret == true)
					return ret; //it is enough that only one is set
			}
			return ret;
		} catch (Exception e) {
			System.out.println(object.eClass().getName() + " " + feature.getName());
		}
		return false;
	}

	@Override
	public void unset(InternalEObject object, EStructuralFeature feature) {
		MergeElementImpl mergeElement = (MergeElementImpl)object;
		VirtualModel vModel = (VirtualModel) object.eResource();
		List<EStructuralFeature> concFeatures = vModel.getMetamodelManager().translateFromMergedFeature(mergeElement, feature);
		for (EStructuralFeature concFeat : concFeatures) {
			EObject concObj = mergeElement.getContainmentElement(concFeat);
			concObj.eUnset(concFeat);
		}
	}

	@Override
	public int hashCode(InternalEObject object, EStructuralFeature feature) {
		//		MergeElementImpl mergeElement = (MergeElementImpl)object;
		//		VirtualModel vModel = (VirtualModel) object.eResource();
		//		EObject cElement = mergeElement.getContainmentElement(feature);
		//		Object value = cElement.eGet(vModel.getMetamodelManager().translateFromVirtualFeature(mergeElement, feature));
		//		return value.hashCode();
		throw new VirtualModelOperationException("hashCode not implemented for merged elements");
	}

	@Override
	public InternalEObject getContainer(InternalEObject object) {
		VirtualModel vModel = (VirtualModel)object.eResource();
		MergeElementImpl mergeElement = (MergeElementImpl)object;
		EObject cElement = mergeElement.getPreferableElement();
		EObject cContainer = cElement.eContainer();
		EObject vContainer = vModel.translateToVirtualElement(cContainer);
		return (InternalEObject) vContainer;
	}

	public EStructuralFeature getContainingFeature(InternalEObject object) {
		VirtualModel vModel = (VirtualModel)object.eResource();
		MergeElementImpl mergeElement = (MergeElementImpl)object;
		EObject cElement = mergeElement.getPreferableElement();
		EStructuralFeature vFeature = vModel.getMetamodelManager().translateToVirtualFeature(object, cElement.eContainingFeature());
		return vFeature;
	}

	@Override
	public EObject create(EClass eClass) {
		throw new VirtualModelOperationException("EStore.move()");
	}

}

